package rice.email.proxy.testing;

import java.io.*;
import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;

import rice.p2p.util.MathUtils;
import rice.p2p.util.SecurityUtils;

public class Hammer {
  static List wordlist;
  
  static HashMap folders;

  public static void createFolder(Folder parent, String name) throws MessagingException {
    Folder bob = parent.getFolder(name);
    if (!bob.exists()) {
      System.out.println("> Create Folder "+name+" in "+parent.getFullName());
      bob.create(Folder.HOLDS_FOLDERS | Folder.HOLDS_MESSAGES);
      folders.put(bob.getFullName(), new ArrayList());
    } else {
      System.out.println("??? trying to create already existing folder");
    }
  }

  public static void createFolder(Session sess, Folder root) throws MessagingException {
    createFolder(selectFolder(root),getWord());
  }
  
  public static void deleteFolder(Folder which) throws MessagingException {
    System.out.println("> Delete Folder "+which.getFullName());
    if (!folders.containsKey(which.getFullName())) {
      System.out.println("!!! folder doesn't exist in local map: "+which.getFullName());
    } else {
      folders.remove(which.getFullName());
      Folder[] children = which.list("*");
      if (children != null) {
	  for (int i = 0; i < children.length; i++) {
	      folders.remove(children[i].getFullName());
	  }
      }
    }
    which.delete(true);
  }
  
  public static void deleteFolder(Session sess, Folder root) throws MessagingException {
    Folder selected = selectFolder(root);
    if (selected != root)
      deleteFolder(selected);
  }

  public static void createMessage(Folder parent, Message msg) throws MessagingException {
    Message[] bob = { msg };
    System.out.println("> Create Message "+msg.toString()+" in "+parent.getFullName());
    parent.appendMessages(bob);
    msg.saveChanges();
    ((List)folders.get(parent.getFullName())).add(messageHash(msg));
  }

  public static void createMessage(Session sess, Folder root) throws MessagingException {
    try {
      Message msg = makeMessage(sess);
      Folder parent = selectFolder(root);
      
      if (parent != root)
        createMessage(parent,msg);
    } catch (UnsupportedEncodingException e) {
      // TODO Auto-generated catch block
      e.printStackTrace(System.out);
    }
  }
  
  public static void deleteMessage(Folder parent, Message msg) throws MessagingException {
    System.out.println("> Delete Message "+msg.toString()+" in "+parent.getFullName());
    msg.setFlag(Flags.Flag.DELETED, true);
    ((List)folders.get(parent.getFullName())).remove(messageHash(msg));
    parent.expunge();
  }
  
  public static void deleteMessage(Session sess, Folder root) throws MessagingException {
    Folder parent = selectFolder(root);
    if (parent == root) {
      System.out.println("! deleteMessage: skipping root folder");
      return;
    }
    parent.open(Folder.READ_WRITE);
    Message m = randomMessage(parent);
    if (m != null) {
      deleteMessage(parent,m);
    }
    parent.close(true);
  }

  public static Message randomMessage(Folder f) throws MessagingException {
    if (f.getMessageCount() > 0) {
      int count = f.getMessageCount();
      int n;
      if (count > 1) 
        n = (new Random()).nextInt(count-1)+1;
      else
        n = 1;
      return f.getMessage(n);
    } else {
      System.out.println("! randomMessage: no messages in folder "+f.getFullName());
      return null;
    }
  }
  
  public static void copyMessage(Folder src, Message msg, Folder target) throws MessagingException {
    System.out.println("> Copy Message "+msg.toString()+" to "+target.getFullName());
    src.copyMessages(new Message[]{msg}, target);
    ((List)folders.get(target.getFullName())).add(messageHash(msg));
  }
  
  public static void copyMessage(Session sess, Folder root) throws MessagingException {
    Folder src = selectFolder(root);
    Folder dst = selectFolder(root);
    if (src.equals(dst)) {
      System.out.println("! copyMessage: skipping no-op copy");
    }
    Message m = randomMessage(src);
    if (m != null) {
      copyMessage(src,m,dst);
    }
    src.open(Folder.READ_ONLY);
  }
  
  public static Folder selectFolder(Folder root) throws MessagingException {
    Folder[] folders = root.list("*");
    
    if (folders == null || folders.length == 0)
      return root;
    
    return folders[(new Random()).nextInt(folders.length)];
  }
  
  public static Message makeMessage(Session sess) throws UnsupportedEncodingException, MessagingException {
    Random r = new Random();
    Message msg = new MimeMessage(sess);
    Address[] froms = { new InternetAddress("bob@example.com","Bob Starfish") };
    msg.addFrom(froms);
    Address[] tos = { new InternetAddress("alice@example.com","Alice Octopus") };
    msg.addRecipients(MimeMessage.RecipientType.TO,tos);
    msg.setSubject(getWords(r,4));
    msg.setSentDate(new Date());
    // n is a chi-squared variable with 3 degrees of freedom and a mean of 3000
    int n;
    do {
      n = (int)((Math.pow(r.nextGaussian(),2) + Math.pow(r.nextGaussian(),2) + Math.pow(r.nextGaussian(),2)) * 1000);
    } while (n < 0 || n > 100000);
    msg.setContent(getWords(r, n),"text/plain");
    // XXX add attachment
    msg.saveChanges();
    return msg;
  }
  
  public static String messageHash(Message m) throws MessagingException {
    ByteArrayOutputStream baos = new ByteArrayOutputStream();
    PrintWriter pw = new PrintWriter(baos);

    try {
      pw.println(m.getSubject());
      pw.println(m.getSentDate());
      pw.println(m.getFrom());
      printAddresses(pw, m.getRecipients(Message.RecipientType.TO));
      printAddresses(pw, m.getRecipients(Message.RecipientType.CC));
      InputStream is = m.getInputStream();
      
      byte[] buf = new byte[1024];
      int n;
      while ((n = is.read(buf)) > 0) {
        baos.write(buf,0,n);
      }
    } catch (IOException e) {
      System.out.println("! messageHash: IOException");
      e.printStackTrace(System.out);
      return null;
    }
    return m.getSubject()+" "+MathUtils.toHex(SecurityUtils.hash(baos.toByteArray()));
  }

  public static void printAddresses(PrintWriter pw, Address[] addresses) {
    if (addresses == null)
      return;
    for (int i = 0; i < addresses.length; i++) {
      pw.println(addresses[i].getType()+": "+addresses[i].toString());
    }
  }
  
  static String getWord(Random r) {
    return (String)wordlist.get(r.nextInt(wordlist.size()));
  }
  
  static String getWords(Random r, int len) {
    StringBuffer sb = new StringBuffer();
    for (int i=0; i<len; i++) {
      if (i != 0) sb.append(" ");
      sb.append(getWord(r));
    }
    return sb.toString();
  }
  
  static String getWord() {
    Random r = new Random();
    return getWord(r);
  }
  
  static String getWords(int len) {
    Random r = new Random();
    return getWords(r,len);
  }
  
  static boolean checkFolder(Folder remote) throws MessagingException {
    remote.open(Folder.READ_ONLY);
    List msgs = new ArrayList(Arrays.asList(remote.getMessages()));
    List local = (List)((ArrayList)folders.get(remote.getFullName())).clone();
    
    Iterator it = msgs.iterator();
    while (it.hasNext() && !msgs.isEmpty() && !local.isEmpty()) {
      String hash = messageHash(((Message)it.next()));
      if (local.contains(hash)) {
        it.remove();
        local.remove(hash);
      }
    }
    
    if (msgs.isEmpty() && local.isEmpty()) {
      remote.close(false);
      return true;
    }
    
    if (!msgs.isEmpty()) {
      System.out.println("!!! checkFolder for "+remote.getFullName());
      System.out.println("  ! remote has messages not in local: ");
      it = msgs.iterator();
      while (it.hasNext()) {
        Message msg = (Message)it.next();
        System.out.println("  ! "+messageHash(msg));
      }
    } 
    
    if (!local.isEmpty()) {
      System.out.println("!!! checkFolder for "+remote.getFullName());
      System.out.println("  ! local has messages not in remote: ");
      it = local.iterator();
      while (it.hasNext()) {
        System.out.println("  ! "+it.next());
      }
    }
    
    remote.close(false);
    return false;
  }
  
  static void checkState(Folder root) {
    try {
      Folder[] allFolders = root.list("*");
      if (allFolders == null) allFolders = new Folder[0];
      HashSet imapFolders = new HashSet(Arrays.asList(allFolders));
      HashMap localFolders = (HashMap)folders.clone();
      boolean fmatch = true;
      
      Iterator it = imapFolders.iterator();
      
      while (it.hasNext() && !imapFolders.isEmpty() && !localFolders.isEmpty()) {
        Folder f = (Folder)it.next();
        if (localFolders.containsKey(f.getFullName())) {
          fmatch &= checkFolder(f);
          it.remove();
          localFolders.remove(f.getFullName());
        }
      }
      
      if (!imapFolders.isEmpty()) {
        System.out.println("!!! checkState remote has folders not in local: ");
        it = imapFolders.iterator();
        while (it.hasNext()) {
          System.out.println("  ! "+((Folder)it.next()).getFullName());
        }
      } 
      
      if (!localFolders.isEmpty()) {
        System.out.println("!!! checkState local has folders not in remote: ");
        it = localFolders.keySet().iterator();
        while (it.hasNext()) {
          System.out.println("  ! "+it.next());
        }
      }
      
      if (fmatch && imapFolders.isEmpty() && localFolders.isEmpty()) {
        System.out.println("checkState passed");
      }

    } catch (MessagingException e) {
      System.out.println("!!! error in checkState");
      e.printStackTrace();
    }
  }

  public static Folder connect(Session session, boolean flush) throws MessagingException {
    // Connect to host
    //
    Store store = session.getStore("imap");
    store.connect("odin12.mpi-sws.mpg.de",1143,"odin12","monkey");
    
    Folder root = store.getDefaultFolder();

    if (flush) {
      Folder[] children = root.list();
      if (children != null) {
        for (int i=0; i< children.length; i++) {
          children[i].delete(true);
        }
      }
      folders = new HashMap();
    }
    
    return root;
  }
  
  /**
   * @param args
   * @throws MessagingException 
   * @throws IOException 
   */
  public static void main(String[] args) throws MessagingException, IOException {
    // load word-list
    BufferedReader re = new BufferedReader(new FileReader("/usr/share/dict/words"));
    String s;
    wordlist = new ArrayList(235000);
    while ((s = re.readLine()) != null) {
      wordlist.add(s);
    }
    re.close();
    
    
    //  Get a Session object
    //
    Properties sysProperties = System.getProperties();
    Session session = Session.getDefaultInstance(sysProperties, null);
    session.setDebug(true);

    Folder root = connect(session,true);
    
    Random r = new Random();
    
    int count = 0;
    
    while (true) {
      try {
        int op = r.nextInt(100);
        if (op < 15) 
          createFolder(session,root); 
        else if (op < 20) 
          deleteFolder(session,root); 
        else if (op < 60) 
          createMessage(session,root); 
        else if (op < 100) 
          deleteMessage(session,root); 
        checkState(root);
        System.out.println(++count + "  -----------------------------------------------------");
      } catch (StoreClosedException sce) {
        sce.printStackTrace(System.out);
        connect(session,false);
      } catch (MessagingException me) {
        me.printStackTrace(System.out);
        if (me.getMessage().indexOf("connection failure") >= 0) {
          connect(session,false);
        }
      }
    }
  }
}
